#!/usr/bin/env python

import yaml
import argparse
from pangeamt_toolkit.processors import Pipeline

parser = argparse.ArgumentParser(description='Preprocess file.')
parser.add_argument('processors', help="Path to yml processors file")
parser.add_argument('data', help="Path to data folder")
parser.add_argument('src', help='Src lang')
parser.add_argument('tgt', help='Tgt lang')

args = parser.parse_args()

langs = [args.src, args.tgt]

for lang in langs:
    processors_file = f'{args.processors}.{lang}.yml'
    with open(processors_file, 'r') as p_file:
        processors = yaml.safe_load(p_file)

    pipeline = Pipeline(processors)
    mods = 'norm.tok.truecase.bpe'
    paths = {
        f'{args.data}/train.{lang}': f'{args.data}/train.{mods}.{lang}',
        f'{args.data}/dev.{lang}': f'{args.data}/dev.{mods}.{lang}',
        f'{args.data}/test.{lang}': f'{args.data}/test.{mods}.{lang}'
    }
    pipeline.preprocess_file(args.src_file, args.tgt_file)



#!/usr/bin/env python

import sys
from pangeamt_toolkit import PangeanmtServer

if len(sys.argv) < 2 or sys.argv[1] == '-h' or sys.argv[1] == '--help':
    print('Syntax: pangeamt_build_server.py <path extended model>'\
        ' <path to save ol model (optional)>')
try:
    server = PangeanmtServer(sys.argv[1], sys.argv[2])
except:
    server = PangeanmtServer(sys.argv[1])

server.start()



from aiohttp import web

async def save(req):
    nmt = req.app['nmt']
    lock = req.app['lock']
    ol = req.app['ol']
    extended_path = req.app['extended_model_path']

    try:
        if not ol:
            raise Exception('Online Learning is not active.')
        async with lock:
            req = await req.json()
            path = ('/').join(extended_path.split('/')[:-1]) + f'_{req['name']}'
            nmt.save_model(path)
            resp = {
                'rc': 0
            }
            return web.json_response(resp, status=200)

    except Exception as e:
        response_obj = {'status': 'failed', 'reason': str(e)}
        return web.json_response(response_obj, status=500)



from aiohttp import web

async def train(req):
    nmt = req.app['nmt']
    pipeline = req.app['pipeline']
    pipeline_tgt = req.app['pipeline_tgt']
    lock = req.app['lock']
    ol = req.app['ol']

    try:
        if not ol:
            raise Exception('Online Learning is not active.')
        req = await req.json() # TODO sensitive point
        async with lock:
            for tu in req['tus']:
                src_preprocessed = pipeline.preprocess_str(tu['src'])
                print(src_preprocessed)
                tgt_preprocessed = pipeline_tgt.preprocess_str(tu['tgt'])
                print(tgt_preprocessed)

                nmt.train(src_preprocessed, tgt_preprocessed)
        resp = {
            'rc': 0
        }
        return web.json_response(resp, status=200)

    except Exception as e:
        response_obj = {'status': 'failed', 'reason': str(e)}
        return web.json_response(response_obj, status=500)



from aiohttp import web as _web

async def translate(req):
    nmt = req.app['nmt']
    pipeline = req.app['pipeline']
    lock = req.app['lock']
    sem = req.app['sem']

    req = await req.json() # TODO sensitive point
    batch_to_trans = []
    async with sem:
        for tu in req.get('tus'):
            src_preprocessed = pipeline.preprocess_str(tu['src'])
            batch_to_trans.append(src_preprocessed)

    async with lock:
        translations = nmt.translate(batch_to_trans)

    for tu in req.get('tus'):
        translation = translations.pop(0)
        translation = (' ').join(translation.tgt)
        tu['tgt'] = pipeline.postprocess_str(translation)

    return _web.json_response(req, status=200)

#    except Exception as e:
#        response_obj = {'status': 'failed', 'reason': str(e)}
#        return _web.json_response(response_obj, status=500)



#!/usr/bin/env python
import yaml
import argparse
from pangeamt_toolkit.processors import Pipeline

parser = argparse.ArgumentParser(description='Preprocess file.')
parser.add_argument('processors', help='Yml file that contains the processes.')
parser.add_argument('src_file', help='Path to src file.')
parser.add_argument('tgt_file', help='Path to tgt file.')

args = parser.parse_args()

with open(args.processors, 'r') as p_file:
    processors = yaml.safe_load(p_file)

pipeline = Pipeline(processors)

pipeline.preprocess_file(args.src_file, args.tgt_file)



import jieba as _jieba

class JiebaTokenizerProcessor:

    @staticmethod
    def tokenize(str, escape=None):
        """ Applies jieba tokenization to str
        """
        return list(_jieba.cut(str))

    @staticmethod
    def detokenize(str_array):
        """ Joins str_array
        """
        return ('').join(str_array)

    @staticmethod
    def preprocess(seg):
        seg.src = JiebaTokenizerProcessor.tokenize(seg.src)

    @staticmethod
    def preprocess_str(seg):
        return JiebaTokenizerProcessor.tokenize(str)

    @staticmethod
    def postprocess(seg):
        seg.tgt = JiebaTokenizerProcessor.detokenize(seg.tgt)

    @staticmethod
    def postprocess_str(str):
        return JiebaTokenizerProcessor.detokenize(str)
